function ModalDialog() {
    this._constructor();
}(function() { // create local scope;

    ModalDialog.BUTTON_OK = "OkButton";
    ModalDialog.BUTTON_CANCEL = "CancelButton";
    ModalDialog.BUTTON_CLOSE = "CloseButton";

    var COVER_ID = "modalCover";
    var DIALOG_ID = "modalDialog";
    var TITLE_ID = "modalDialogTitle";
    var CONTENT_ID = "modalDialogContentPane";
    var BUTTONS_BAR_ID = "modalDialogButtonsBar";
    var Z_INDEX = 0x7FFFFE00;
    var DEFAULT_OPACITY = 0.75;
    var TABBED_TAGS = ["A", "AREA", "BUTTON", "INPUT", "OBJECT", "SELECT", "TEXTAREA"];

    ModalDialog.prototype = {

        shim: null,
        cover: null,
        dialog: null,
        isIENavigator: null,

        _constructor: function() {
            this.isIENavigator = /MSIE\s*(5\.5|6\.)/.test(navigator.userAgent);
            this.shim = this._createShim();
            this.cover = this._createCover();
            this.dialog = this._createDialog();
            this.setTitle(null);
            this.setOpacity(DEFAULT_OPACITY);

            var thisObj = this;
            window.onresize = function() { thisObj.update(); }
            this.update();
        },

        addButton: function(type, text, handler) {
            var button = Utils.createInput(Tag.INPUT_TYPE_BUTTON, type);
            button.value = text;

            var bar = $(BUTTONS_BAR_ID);
            bar.appendChild(document.createTextNode(String.fromCharCode(160)));
            bar.appendChild(button);

            var thisObj = this; 
            button.onclick = function() {
                if (typeof handler == Type.FUNCTION) {
                    handler(this);
                } else {
                    thisObj._handleButtonPressed(this);
                }
            }
        },

        getContent: function() {
            return $(CONTENT_ID);
        },

        setVisible: function(visible) {
            var value = visible ? "block" : "none";
            this.cover.style.display = value;
            this.dialog.style.display = value;
            this._requestFocus();

            if (this.shim != null) {
                this.shim.style.display = value;
            }

            if (this.isIENavigator) {
                document.getElementsByTagName("html")[0].style.overflow = visible ? "hidden" : "";
                if (visible) {
                    this._disableTabbedTags();
                } else {
                    this._restoreTabbedTags();
                }
            }
        },

        setTitle: function(value) {
            var node = document.createElement(Tag.DIV);
            node.id = TITLE_ID;
            node.appendChild(document.createTextNode(value || document.title));
            
            var title = $(TITLE_ID);

            if (title != null) {
                this.dialog.replaceChild(node, title);
            } else {
                this.dialog.insertBefore(node, this.getContent());
            }
        },

        setOpacity: function(value) {
            Utils.setOpacity(this.cover, value || DEFAULT_OPACITY);
        },

        setCssClass: function(value) {
            Utils.addCssClass(this.dialog, value || '');
        },

        _createShim: function() {
            if (this.isIENavigator && document.body.insertAdjacentHTML) {
                var html = "<iframe src='about:blank' frameborder='0' scrolling='no' "
                        + "style='position:absolute;display:none;filter:alpha(opacity:0);z-index:" + (Z_INDEX - 1) + "'></iframe>"
                document.body.insertAdjacentHTML("AfterBegin", html);
                return document.body.firstChild;
            }

            return null;
        },

        _createCover: function() {
            var result = $(COVER_ID);

            if (result == null) {
                result = document.body.appendChild(document.createElement(Tag.DIV));
                result.id = COVER_ID;
                Utils.setStyle(result, this.isIENavigator
                                       ? {position: "absolute", display: "none", zIndex: Z_INDEX}
                                       : {position: "fixed", top: 0, left: 0, width: "100%", height: "100%", zIndex: Z_INDEX});
            }

            return result;
        },

        _createDialog: function() {
            var result = $(DIALOG_ID);

            if (result == null) {
                result = document.body.appendChild(document.createElement(Tag.DIV));
                result.id = DIALOG_ID;
                Utils.setStyle(result, this.isIENavigator
                                        ? {position: "absolute", display: "none", zIndex: Z_INDEX + 1}
                                        : {position: "fixed", top: 0, left: 0, zIndex: Z_INDEX + 1});

                var content = document.createElement(Tag.DIV);
                content.id = CONTENT_ID;
                result.appendChild(content);

                var buttonsBar = document.createElement(Tag.DIV);
                buttonsBar.id = BUTTONS_BAR_ID;
                result.appendChild(buttonsBar);
            }

            return result;
        },

        update: function() {
            //todo: check Safari dialog center position
            var viewportDimension = document.viewport.getDimensions();
            var dialogDimension = Utils.getDimension(this.dialog);
            var dialogTop = Math.max(0, (viewportDimension.height - dialogDimension.height) / 2 - viewportDimension.height * 0.15);
            var dialogLeft = Math.max(0, (viewportDimension.width - dialogDimension.width) / 2);

            if (this.isIENavigator) {
                var viewportScrollOffsets = document.viewport.getScrollOffsets();
                var coverPosition = {top: viewportScrollOffsets.top + 'px', left: viewportScrollOffsets.left + 'px',
                                     width: viewportDimension.width + 'px', height: viewportDimension.height + 'px'};
                if (this.shim != null) {
                    Utils.setStyle(this.shim, coverPosition);
                }

                Utils.setStyle(this.cover, coverPosition);
                Utils.setStyle(this.dialog, {top: viewportScrollOffsets.top + dialogTop + 'px', left: viewportScrollOffsets.left + dialogLeft + 'px'});
            } else {
                Utils.setStyle(this.dialog, {top: dialogTop + 'px', left: dialogLeft + 'px'});
            }
        },

        onSubmit: function() {
        },

        onCancel: function() {
        },

        _requestFocus: function() {
            if (Utils.isVisible(this.dialog)) {
                var buttons = $(BUTTONS_BAR_ID).getElementsByTagName(Tag.INPUT);
                var el = buttons.length > 0 ? buttons[0] : this.getContent();
                try {
                    el.focus();
                } catch (e) {}
            }
        },

        _handleButtonPressed: function(button) {
            if (button.name == ModalDialog.BUTTON_OK) {
                this.onSubmit();
            } else if (button.name == ModalDialog.BUTTON_CANCEL) {
                this.onCancel();
            } else if (button.name == ModalDialog.BUTTON_CLOSE) {
                this.setVisible(false)
            }
        },

        _disableTabbedTags: function() {
            var elements = this._getTabbedElements(document);
            var excludeElements = this._getTabbedElements(this.dialog);
            
            for (var i = 0; i < elements.length; ++i) {
                var el = elements[i];
                if (excludeElements.indexOf(el) == -1) {
                    el.originalDisabled = el.disabled;
                    el.disabled = true;
                }
            }
        },

        _restoreTabbedTags: function() {
            var elements = this._getTabbedElements(document);
            var excludeElements = this._getTabbedElements(this.dialog);

            for (var i = 0; i < elements.length; ++i) {
                var el = elements[i];
                if (excludeElements.indexOf(el) == -1) {
                    if (typeof el.originalDisabled != Type.UNDEFINED) {
                        el.disabled = el.originalDisabled;
                    }
                }
            }
        },

        _getTabbedElements: function(parent) {
            var result = [];
            for (var i = 0; i < TABBED_TAGS.length; ++i) {
                var tags = parent.getElementsByTagName(TABBED_TAGS[i]);
                for (var j = 0; j < tags.length; ++j) {
                    result.push(tags[j]);
                }
            }
            return result;
        }
    }
})();